"use strict";
require("reflect-metadata");
var plan_1 = require("./plan");
var context_1 = require("./context");
var request_1 = require("./request");
var target_1 = require("./target");
var METADATA_KEY = require("../constants/metadata_keys");
var ERROR_MSGS = require("../constants/error_msgs");
var Planner = (function () {
    function Planner() {
    }
    Planner.prototype.createContext = function (kernel) {
        return new context_1.Context(kernel);
    };
    Planner.prototype.createPlan = function (context, binding) {
        var _this = this;
        var rootRequest = new request_1.Request(binding.runtimeIdentifier, context, null, binding);
        rootRequest.bindings.push(binding);
        var plan = new plan_1.Plan(context, rootRequest);
        var dependencies = this._getDependencies(binding.implementationType);
        dependencies.forEach(function (d) {
            _this._createSubRequest(rootRequest, d);
        });
        return plan;
    };
    Planner.prototype._createSubRequest = function (parentRequest, target) {
        var _this = this;
        var bindings = this._getBindings(parentRequest.parentContext, target.service.value());
        if (bindings.length > 1) {
            throw new Error(ERROR_MSGS.AMBIGUOUS_MATCH + " " + target.service.value());
        }
        else {
            var binding = bindings[0];
            var childRequest_1 = parentRequest.addChildRequest(target.service.value(), binding, target);
            var subDependencies = this._getDependencies(binding.implementationType);
            subDependencies.forEach(function (d, index) {
                _this._createSubRequest(childRequest_1, d);
            });
        }
    };
    Planner.prototype._getBindings = function (context, service) {
        var kernel = context.kernel;
        var bindingDictionary = kernel._bindingDictionary;
        return bindingDictionary.get(service);
    };
    Planner.prototype._getDependencies = function (func) {
        var injections = Reflect.getMetadata(METADATA_KEY.INJECT, func) || [];
        var paramNames = Reflect.getMetadata(METADATA_KEY.PARAM_NAMES, func) || [];
        var tags = Reflect.getMetadata(METADATA_KEY.TAGGED, func) || [];
        return injections.map(function (inject, index) {
            var targetName = paramNames[index];
            var target = new target_1.Target(targetName, inject);
            target.metadata = tags[index.toString()] || [];
            return target;
        });
    };
    return Planner;
}());
exports.Planner = Planner;
